package main

import modell.GameModell
import java.io.Console
import scala.swing._
import javax.swing.BorderFactory
import java.awt.Color
import scala.swing.event.KeyPressed
import scala.swing.event.Key
import scala.swing.event.KeyTyped
import scala.swing.event.MouseClicked
import scala.swing.event.KeyPressed
import scala.swing.Label
import modell.Direction
import modell.Direction._

//Java Swinghez hasonlóan a Scala Swing esetén is az ablakok a 
//MainFrame-ből származnak
class UI extends MainFrame {
  val modell = new GameModell()
  title = "2048"
  preferredSize = new Dimension(320, 240)

  val colors = Map(
    0 -> Color.white,
    2 -> Color.gray,
    4 -> Color.blue,
    8 -> Color.orange,
    16 -> Color.green,
    32 -> Color.pink,
    64 -> Color.red,
    128 -> Color.cyan,
    256 -> Color.magenta,
    512 -> Color.yellow)

  contents = new GridPanel(4, 4) {
    var labels = new Array[Label](16)
    for (i <- 0 to labels.size - 1) {
      labels(i) = new Label
      labels(i).border = BorderFactory.createLineBorder(Color.black)
      labels(i).opaque = true
      contents += labels(i)
    }
    refresh

    focusable = true
    listenTo(keys)
    reactions += {
      case KeyPressed(_, Key.Up, _, _) =>
        step(Direction.UP)
      case KeyPressed(_, Key.Down, _, _) =>
        step(Direction.DOWN)
      case KeyPressed(_, Key.Right, _, _) =>
        step(Direction.RIGHT)
      case KeyPressed(_, Key.Left, _, _) =>
        step(Direction.LEFT)
    }

    //a modellből lekéri a tábla minden eleméhez tartozó számértéket
    //és ez alapján színezi a táblát
    def refresh = {
      for (i <- 0 to 3)
        for (j <- 0 to 3) {
          val num = modell.get(i, j)
          labels(i * 4 + j).text = num.toString()
          labels(i * 4 + j).background = colors(num)
        }
    }

    //az egy lépéshez tartozó eljárás 
    //lépteti a modellt, lerak egy új elemet 
    //és meghívja a refresh fv-t
    def step(dir: Direction) = {
      modell.step(dir)
      modell.addNew
      refresh
    }
  }
}

//a Main objektumot az object kulcsszóval deklarálva singletont kapunk
object Main {
  def main(args: Array[String]) {
    val ui = new UI
    ui.visible = true
  }
}