package modell

import scala.util.Random
import modell.Direction._

class GameModell {
  //az itt látható kód a GameModell példányosításakor lefut,
  //olyan mintha konstruktorba raktuk volna az utasításokat
  
  val BACKWARD = -1
  val FORWARD = 1

  val size = 4;
  
  //egy size * size méretű tömb, ami kezdetben 0-kkal van feltöltve
  val board = Array.fill(size)(Array.fill(size)(0))

  //már a GameModell példányosításakor szükség van az addNew fv hívására
  addNew

  //ha a tábla még nincs tele akkor generál két random indexet
  //és arra a pozícióra rak le egy 2-es számot
  //ha a generált index már foglalt, akkor újat generál
  def addNew = {
    if (!isFull) {
      val r = new Random
      //többszörös értékadás egy tuple használatával
      var (i, j) = (r.nextInt(size), r.nextInt(size))
      while (board(i)(j) != 0) {
        i = r.nextInt(size)
        j = r.nextInt(size)
      }
      board(i)(j) = 2
    }
  }

  //megvizsgáljuk, hogy a táblán minden mezőnek nagyobb értéke van-e mint 0
  def isFull: Boolean = {
    for (i <- 0 to size - 1)
      if (board(i) contains (0))
        return false
    true
  }

  //egy lépést végző fv
  //megkapja, hogy melyik irányba kell a táblán lévő mezőket léptetni
  //eszerint paraméterezi és hívja a megfelelő függvényt
  def step(direction: Direction) =
    direction match {
      case Direction.UP => moveVertical(1, size - 1, BACKWARD)
      case Direction.DOWN => moveVertical(size, 0, FORWARD)
      case Direction.RIGHT => moveHorizontal(size, 0, FORWARD)
      case _ => moveHorizontal(1, size - 1, BACKWARD)
    }

  def moveVertical(from: Int, to: Int, back: Int) =
    for (i <- 0 to size - 1)
      for (j <- from to to by -back)
        moveElement(j, i, back, 0)

  def moveHorizontal(from: Int, to: Int, back: Int) =
    for (i <- 0 to size - 1)
      for (j <- from to to by -back)
        moveElement(i, j, 0, back)

  //egy elem mozgatása
  def moveElement(startX: Int, startY: Int, dirX: Int, dirY: Int) = {
    var (x, y) = (startX + dirX, startY + dirY)
    while (isOnBoard(x, y) && isFieldEmpty(x, y)) {
      swap(x, y, x - dirX, y - dirY)
      x += dirX
      y += dirY
    }
    if (isOnBoard(x, y)
      && board(x)(y) == board(x - dirX)(y - dirY)) {
      board(x)(y) *= 2
      board(x - dirX)(y - dirY) = 0
    }
  }

  //fv túlterhelés különböző paraméterekkel
  def isOnBoard(c: Int) = c <= size - 1 && c >= 0
  def isOnBoard(x: Int, y: Int): Boolean = isOnBoard(x) && isOnBoard(y)

  def isFieldEmpty(x: Int, y: Int) = board(x)(y) == 0

  def get(x: Int, y: Int) = board(x)(y)

  def swap(nx1: Int, ny1: Int, vx2: Int, vy2: Int) = {
    board(nx1)(ny1) = board(vx2)(vy2)
    board(vx2)(vy2) = 0
  }
}